package com.tzld.longarticle.recommend.server.service.recommend.score.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleTypeEnum;
import com.tzld.longarticle.recommend.server.remote.NLPRemoteService;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.service.recommend.score.Score;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Component
@Slf4j
public class SimilarityStrategy implements ScoreStrategy {

    @Autowired
    private NLPRemoteService nlpRemoteService;
    @Autowired
    private ArticleRepository articleRepository;

    @ApolloJsonValue("${account.score.sim.replace:{}}")
    private Map<String, String> accountSimScoreReplaceMap;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }
        String ghId = param.getGhId();
        if (accountSimScoreReplaceMap.containsKey(ghId)) {
            int historyCount = articleRepository.countByGhIdAndTypeAndItemIndex(ghId,
                    ArticleTypeEnum.QUNFA.getVal(), 1);
            if (historyCount < 10) {
                ghId = accountSimScoreReplaceMap.get(ghId);
            }
        }
        Map<String, Double> scoreMap = nlpRemoteService.score(ghId, param.getAccountName(), param.getContents());

//        double min = scoreMap.values().stream()
//                .min(Double::compareTo)
//                .orElse(0.0);
//        double max = scoreMap.values().stream()
//                .max(Double::compareTo)
//                .orElse(0.0);

        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            double val = scoreMap.get(c.getId()) == null ? 0.0 : scoreMap.get(c.getId());
            // score.setScore(NormalizationUtils.minMax(val, min, max));
            score.setScore(val);
            score.setStrategy(this);
            return score;
        });
        log.info("SimilarityStrategy cost:{}", System.currentTimeMillis() - start);
        return scores;
    }

}
